package dk.nsp.cbs.perf.runner;

import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;
import java.util.function.Predicate;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import com.google.common.collect.ImmutableList;

public class Splitter {

  private static<T> Predicate<T> not(Predicate<T> inner) {
    return t -> !inner.test(t);
  }

  public static void main(String[] args) throws IOException {
    final Path totalFile = Paths.get(args[0]);
    final Path hostsFile = Paths.get(args[1]);

    if (!Files.exists(totalFile) || !Files.exists(hostsFile)) {
      System.err.printf("Both %s and %s must exist.", totalFile, hostsFile);
      System.exit(-1);
    }

    output(totalFile, hostsFile);
  }

  public static<T> Stream<T> take(Iterator<T> ts, long count) {
    final Iterable<T> iterable = () -> ts;
    return StreamSupport.stream(iterable.spliterator(), false).limit(count);
  }

  static void output(Path totalFile, Path hostsFile) throws IOException {
    final ImmutableList<String> hosts = Files.lines(hostsFile)
                                               .filter(not(String::isEmpty))
                                               .collect(ImmutableList.toImmutableList());

    final long nodes = hosts.size();
    final long cprCount = Files.lines(totalFile)
                               .filter(not(String::isEmpty))
                               .count();
    final long perNodes = cprCount / nodes;
    System.out.printf("Putting %d cpr numbers into each file [hosts=%d, cpr=%d].\n", perNodes, nodes, cprCount);

    try(final Stream<String> cprStream = Files.lines(totalFile)
                                              .filter(not(String::isEmpty)) ) {

      final Iterator<String> cprIterator = cprStream.iterator();

      hosts.stream()
           .map(host -> totalFile.getParent().resolve(String.format("%s-cpr.txt", host)))
           .forEach(hostCprFile -> {
             try (PrintStream out = new PrintStream(Files.newOutputStream(hostCprFile))) {
               take(cprIterator, perNodes)
                 .forEach(out::println);
             } catch (IOException e) {
               throw new RuntimeException(e);
             }
           });
    }
  }
}
