package dk.nsp.cbs.perf.stub;

import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.advis.v10.NotifyContent;
import org.oasis_open.docs.wsn.b_2.CreatePullPoint;
import org.oasis_open.docs.wsn.b_2.CreatePullPointResponse;
import org.oasis_open.docs.wsn.b_2.GetMessages;
import org.oasis_open.docs.wsn.b_2.GetMessagesResponse;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;
import org.oasis_open.docs.wsn.b_2.Subscribe;
import org.oasis_open.docs.wsn.b_2.SubscribeResponse;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3._2005._08.addressing.AttributedURIType;
import org.w3._2005._08.addressing.EndpointReferenceType;
import org.xmlsoap.schemas.soap.envelope.Body;
import org.xmlsoap.schemas.soap.envelope.Envelope;

public class NasProtocol {

  private static final org.oasis_open.docs.wsn.b_2.ObjectFactory WSN_B2_FACTORY = new org.oasis_open.docs.wsn.b_2.ObjectFactory();
  private static final org.xmlsoap.schemas.soap.envelope.ObjectFactory SOAP_FACTORY = new org.xmlsoap.schemas.soap.envelope.ObjectFactory();

  private final JAXBContext jaxbContext;

  {
    try {
      this.jaxbContext = JAXBContext.newInstance(Envelope.class,
                                                 Security.class,
                                                 Header.class,
                                                 CreatePullPoint.class,
                                                 Subscribe.class,
                                                 GetMessages.class,
                                                 NotifyContent.class);
    } catch (JAXBException e) {
      System.out.println("Could not create jaxbContext!!!"+ e);
      throw new RuntimeException("Bad things are happening");
    }
  }

  private String makeAnswer() throws JAXBException {
    final CreatePullPointResponse response = WSN_B2_FACTORY.createCreatePullPointResponse();
    final EndpointReferenceType pullpoint = new EndpointReferenceType();
    final AttributedURIType value = new AttributedURIType();
    value.setValue("spongebob"); // should be the full point URL for the pull point
    pullpoint.setAddress(value);
    response.setPullPoint(pullpoint);

    final Envelope envelope = SOAP_FACTORY.createEnvelope();
    final Body body = SOAP_FACTORY.createBody();
    envelope.setBody(body);

    body.getAny().add(response);

    StringWriter requestXMLWriter = new StringWriter();
    jaxbContext.createMarshaller().marshal(envelope, requestXMLWriter);
    String requestXml = requestXMLWriter.toString();
    return requestXml;
  }

  private final NasServer nasServer;

  public NasProtocol(NasServer nasServer) {
    this.nasServer = nasServer;
  }


  @SuppressWarnings("unchecked")
  private <T> T parse(String body) {
    try {
      final Envelope env = (Envelope) jaxbContext.createUnmarshaller()
                                                 .unmarshal(new StringReader(body));
      return (T) env.getBody().getAny().get(0);
    } catch (JAXBException e) {
      throw new RuntimeException(e);
    }
  }

  private String marshal(Object thing) {
    final Envelope envelope = SOAP_FACTORY.createEnvelope();
    final Body responseBody = SOAP_FACTORY.createBody();
    envelope.setBody(responseBody);
    responseBody.getAny().add(thing);
    final StringWriter requestXMLWriter = new StringWriter();
    try {
      jaxbContext.createMarshaller().marshal(envelope, requestXMLWriter);
    } catch (JAXBException e) {
      throw new RuntimeException(e);
    }
    return requestXMLWriter.toString();
  }


  public String pullPointFactory(String body) {
    // TODO: verify this is the only call
    final CreatePullPoint createPullPoint = parse(body);

    final CreatePullPointResponse response = WSN_B2_FACTORY.createCreatePullPointResponse();
    final EndpointReferenceType pullpoint = new EndpointReferenceType();
    final AttributedURIType value = new AttributedURIType();
    value.setValue(nasServer.createPullPoint()); // should be the full point URL for the pull point
    pullpoint.setAddress(value);
    response.setPullPoint(pullpoint);

    return marshal(response);
  }

  public String pullpoint(String body) {
    final GetMessages getMessages = parse(body);

    final GetMessagesResponse response = WSN_B2_FACTORY.createGetMessagesResponse();
    // for all jobId
    for (String jobId : nasServer.getJobIds()) {
      final NotificationMessageHolderType msg = new NotificationMessageHolderType();
      final NotificationMessageHolderType.Message value = new NotificationMessageHolderType.Message();
      final NotifyContent value1 = new NotifyContent();
      value1.setId(jobId);
      value.setAny(value1);
      msg.setMessage(value);
      response.getNotificationMessage().add(msg);
    }

    return marshal(response);
  }

  public String subscriptionManager(String body) {
    final Subscribe subscribeReq = parse(body);

    // String subscriptionId = subscribeResponse.getSubscriptionReference().getAddress().getValue();
    final SubscribeResponse subscribeResponse = WSN_B2_FACTORY.createSubscribeResponse();
    final EndpointReferenceType value = new EndpointReferenceType();
    final AttributedURIType value1 = new AttributedURIType();
    value1.setValue(nasServer.createSubscription());
    value.setAddress(value1);
    subscribeResponse.setSubscriptionReference(value);

    return marshal(subscribeResponse);
  }
}
